#!/usr/bin/env python
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: BSD-2-Clause
'''keygone.py -- A cruddy improvised NTPsec keyfile generator

The number in the group_iterator range is the number of keys to generate
per key type.

key_number starts as the first key number and is incremented.

max_bytes is the implementation-specific maximum number of entropy bytes
allowed per key.

types_lengths_list is a list of lists. the inner lists store the output
length of the algorithm (block size?) and the standard form of the
algorithms name (in lowercase). Commented out algorithms are believed
broken, non-commented ones are not. This list believed accurate as of
the 6th of May 2020.
'''
import secrets

group_iterator = range(10)
key_number = 100
max_bytes = 32

types_lengths_list = [
    # [16, 'md5'],
     [16, 'aes-128'],
     [16, 'aria-128'],
     [16, 'camellia-128'],
     [16, 'sm4'],

    # [20, 'sha1'],
    # [20, 'rmd160'],

     [24, 'aes-192'],
     [24, 'aria-192'],
     [24, 'camellia-192'],

     [28, 'sha224'],
     [28, 'sha3-224'],

     [32, 'blake2s256'],
     [32, 'sha256'],
     [32, 'sha3-256'],
     [32, 'aes-256'],
     [32, 'aria-256'],
     [32, 'camellia-256'],

     [48, 'sha384'],
     [48, 'sha3-384'],

     [64, 'blake2b512'],
     [64, 'sha512'],
     [64, 'sha3-512']]

for type_length_tuple in types_lengths_list:
    length = min(type_length_tuple[0], max_bytes)
    for _ in group_iterator:
        print("%3d %12s\t%s" %
        (key_number, type_length_tuple[1],
         secrets.token_hex(length)))
        key_number += 1



def options_cmd(ctx, config):
    ctx.load("compiler_c")
    ctx.load("msvc")
    ctx.load('waf_unit_test')
    ctx.load('gnu_dirs')

    def callback_flags(option, opt, value, parser):
        config["OPT_STORE"].setdefault(opt, []).append(value)

    grp = ctx.add_option_group("NTP configure options")
    grp.add_option('--enable-debug', action='store_true',
                   default=False, help="Enable debugging code")
    grp.add_option('--enable-debug-gdb', action='store_true',
                   default=False, help="Enable GDB debugging symbols")
    grp.add_option('--disable-nts', action='store_true',
                   default=False, help="Disable NTS.")
    grp.add_option('--disable-droproot', action='store_true',
                   default=False, help="Disable dropping root.")
    grp.add_option('--enable-early-droproot', action='store_true',
                   default=False,
                   help="Droproot earlier (breaks SHM and NetBSD).")
    grp.add_option('--enable-seccomp', action='store_true',
                   default=False, help="Enable seccomp (restricts syscalls).")
    grp.add_option('--disable-mdns-registration', action='store_true',
                   default=False, help="Disable MDNS registration.")
    grp.add_option(
        '--enable-classic-mode', action='store_true',
        default=False,
        help="Strict configuration and log-format "
             " compatibility with NTP Classic")
    grp.add_option('--enable-debug-timing', action='store_true',
                   default=False,
                   help="Collect timing statistics for debugging.")

    grp = ctx.add_option_group("NTP cross compile options")
    grp.add_option('--cross-compiler', type='string',
                   help="Path to cross compiler CC. (enables cross-compiling)")
    grp.add_option('--cross-cflags', type='string',  action="callback",
                   callback=callback_flags, help="Cross compiler CFLAGS.")
    grp.add_option('--cross-ldflags', type='string', action="callback",
                   callback=callback_flags, help="Cross compiler LDFLAGS.")

    grp = ctx.add_option_group("NTP configure features")
    grp.add_option('--enable-leap-smear', action='store_true',
                   default=False, help="Enable Leap Smearing.")
    grp.add_option('--enable-leap-testing', action='store_true',
                   default=False,
                   help="Enable leaps on other than 1st of month.")
    grp.add_option('--enable-mssntp', action='store_true',
                   default=False, help="Enable Samba MS SNTP support.")

    grp = ctx.add_option_group("Refclock configure options")
    grp.add_option(
        '--refclock', dest='refclocks',
        help="Comma-separated list of Refclock IDs to build (or \"all\")",
        type='string')
    grp.add_option('--list', action='store_true', default=False,
                   help="List available Refclocks")

    grp = ctx.add_option_group("NTP developer configure options")
    grp.add_option('--build-version-tag', type='string',
                   help="Append a tag to the version string (unused)")
    grp.add_option('--cflags', type='string', action="callback",
                   callback=callback_flags,
                   help="Users should use CFLAGS in their environment.")
    grp.add_option('--build-epoch', type='int', default=None,
                   help="Force epoch, or use SOURCE_DATE_EPOCH in environment")
    grp.add_option('--ldflags', type='string', action="callback",
                   callback=callback_flags,
                   help="Users should use LDFLAGS in their environment.")
    grp.add_option('--check', action='store_true', default=False,
                   help="Run tests")
    grp.add_option('--enable-rtems-trace', action='store_true',
                   default=False, help="Enable RTEMS Trace.")
    grp.add_option('--enable-warnings', action='store_true',
                   default=False, help="Enable annoying CC warnings")
    grp.add_option('--rtems-trace-path', type='string', default="",
                   help="Path to rtems-tld.")
    grp.add_option(
        '--define', type='string', action="callback",
        callback=callback_flags,
        help="Force definition of symbol, wants value of form SYM=VAL.")
    grp.add_option('--undefine', type='string', action="callback",
                   callback=callback_flags,
                   help="Force undefinition of symbol.")
    grp.add_option('--disable-fuzz', action='store_true',
                   default=False,
                   help="Disable fuzzing of low bits of time")



from __future__ import print_function

from waflib.Logs import pprint


def test_write_log(ctx):
    file_out = "%s/test.log" % ctx.bldnode.abspath()

    log = getattr(ctx, 'utest_results', [])

    if not log:
        return

    with open(file_out, "w") as fp:
        for binary, retval, lines, error in ctx.utest_results:
            fp.write("BINARY      : %s\n" % binary)
            fp.write("RETURN VALUE: %s\n" % retval)
            fp.write("\n*** stdout ***\n")
            fp.write(str(lines))
            fp.write("\n*** stderr ***\n")
            fp.write(str(error))
            fp.write("\n\n\n")

    pprint("BLUE", "Wrote test log to: ", file_out)


def test_print_log(ctx):
    for binary, retval, lines, error in ctx.utest_results:
        pprint("YELLOW", "BINARY      :", binary)
        pprint("YELLOW", "RETURN VALUE:", retval)
        print("")

        if retval or error:
            pprint("RED", "****** ERROR ******\n")

            print(error or lines)

        if (not retval) and (not error):
            pprint("GREEN", "****** LOG ******\n", lines)

        print("")



"""Run a suite of tests on the listed binaries."""
from __future__ import print_function
import os
import os.path
import sys
import waflib.Context
import waflib.Logs
import waflib.Utils
sys.path.insert(0, "%s/main/tests/pylib" % waflib.Context.out_dir)
import ntp.poly
import ntp.util

verStr = ntp.util.stdversion()

cmd_map = {
    ("main/ntpclients/ntpleapfetch", "--version"): "ntpleapfetch %s\n"
                                                   % verStr,
    ("main/ntpd/ntpd", "--version"): "ntpd %s\n" % verStr,
    ("main/ntpfrob/ntpfrob", "-V"): "ntpfrob %s\n" % verStr,
    ("main/ntptime/ntptime", "-V"): "ntptime %s\n" % verStr
}
cmd_map_python = {
    ("main/ntpclients/ntpdig", "--version"): "ntpdig %s\n" % verStr,
    ("main/ntpclients/ntpkeygen", "--version"): "ntpkeygen %s\n" % verStr,
    ("main/ntpclients/ntpq", "--version"): "ntpq %s\n" % verStr,
    ("main/ntpclients/ntpsnmpd", "--version"): "ntpsnmpd %s\n" % verStr,
    ("main/ntpclients/ntpsweep", "--version"): "ntpsweep %s\n" % verStr,
    ("main/ntpclients/ntptrace", "--version"): "ntptrace %s\n" % verStr,
    ("main/ntpclients/ntpwait", "--version"): "ntpwait %s\n" % verStr
}
# Need argparse
cmd_map_python_argparse = {
    ("main/ntpclients/ntplogtemp", "--version"): "ntplogtemp %s\n" % verStr,
    ("main/ntpclients/ntpviz", "--version"): "ntpviz %s\n" % verStr,
}
# Need python curses
cmd_map_python_curses = {
    ("main/ntpclients/ntpmon", "--version"): "ntpmon %s\n" % verStr,
}

test_logs = []


def addLog(color, text):
    test_logs.append((color, text))


def bin_test_summary(ctx):
    for i in test_logs:
        waflib.Logs.pprint(i[0], i[1])


def run(cmd, reg, pythonic):
    """Run an individual non-python test."""
    check = False

    breg = ntp.poly.polybytes(reg)

    prefix = "running: " + " ".join(cmd)

    if not os.path.exists("%s/%s" % (waflib.Context.out_dir, cmd[0])):
        addLog("YELLOW", prefix + " SKIPPING (does not exist)")
        return False

    if pythonic:
        cmd = [sys.executable] + list(cmd)
    p = waflib.Utils.subprocess.Popen(cmd, env={'PYTHONPATH': '%s/main/tests/pylib' %
                                                 waflib.Context.out_dir},
                         stdin=waflib.Utils.subprocess.PIPE,
                         stdout=waflib.Utils.subprocess.PIPE,
                         stderr=waflib.Utils.subprocess.PIPE, cwd=waflib.Context.out_dir)

    stdout, stderr = p.communicate()

    if (stdout == breg) or (stderr == breg):
        check = True

    if check:
        addLog("GREEN", prefix + "  OK")
        return True
    addLog("RED", prefix + "  FAILED")
    addLog("PINK", "Expected: " + breg)
    if stdout:
        addLog("PINK", "Got (stdout): " + ntp.poly.polystr(stdout))
    if stderr:
        addLog("PINK", "Got (stderr): " + ntp.poly.polystr(stderr))
    return False


def cmd_bin_test(ctx, config):
    """Run a suite of binary tests."""
    fails = 0

    if ctx.env['PYTHON_ARGPARSE']:
        cmd_map_python.update(cmd_map_python_argparse)

    if ctx.env['PYTHON_CURSES']:
        cmd_map_python.update(cmd_map_python_curses)

    for cmd in sorted(cmd_map):
        if not run(cmd, cmd_map[cmd], False):
            fails += 1

    for cmd in sorted(cmd_map_python):
        if not run(cmd, cmd_map_python[cmd], True):
            fails += 1

    if 1 == fails:
        bin_test_summary(ctx)
        ctx.fatal("1 binary test failed!")
    elif 1 < fails:
        bin_test_summary(ctx)
        ctx.fatal("%d binary tests failed!" % fails)



SNIP_LIBSSL_TLS13_CHECK = """
#include <openssl/tls1.h>

#ifndef TLS1_3_VERSION
#error OpenSSL must have support for TLSv1.3
#endif

int main(void) {
    return 0;
}
"""


def check_libssl_tls13(ctx):
    ctx.check_cc(
      fragment=SNIP_LIBSSL_TLS13_CHECK,
      use="SSL",
      msg="Checking for OpenSSL with TLSv1.3 support",
    )


SNIP_OPENSSL_BAD_VERSION_CHECK = """
#include <openssl/opensslv.h>

#if OPENSSL_VERSION_NUMBER == 0x1010101fL
#error OpenSSL version must not be 1.1.1a
#endif

int main(void) {
    return 0;
}
"""


def check_openssl_bad_version(ctx):
    ctx.check_cc(
      fragment=SNIP_OPENSSL_BAD_VERSION_CHECK,
      use="SSL",
      msg="Checking for OpenSSL != 1.1.1a",
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# SPDX-License-Identifier: BSD-2-Clause
import unittest
import ntp.ntpc


class TestPylibNtpc(unittest.TestCase):
    lfp_set = [
        ("0xcfba1ce0.80000000", 1276092000.5,
         "cfba1ce0.80000000 2010-06-09T14:00:00.500Z"),
        ("==cfba1ce0.80000000", 1276092000.5,
         "cfba1ce0.80000000 2010-06-09T14:00:00.500Z"),
        ]

    def test_statustoa(self):
        self.assertEqual("leap_add_sec, sync_22, 7 events, no_sys_peer",
                         ntp.ntpc.statustoa(ntp.ntpc.TYPE_SYS, 0x12345678));
        self.assertEqual("authenb, reach, sel_sys.peer, 7 events, access_denied",
                         ntp.ntpc.statustoa(ntp.ntpc.TYPE_PEER, 0x12345678));
        self.assertEqual("7 events, clk_8",
                         ntp.ntpc.statustoa(ntp.ntpc.TYPE_CLOCK, 0x12345678));

    def test_lfp(self):
        for (in_string, to_float, to_string) in self.lfp_set:
            self.assertEqual(ntp.ntpc.prettydate(in_string), to_string);
            self.assertAlmostEqual(ntp.ntpc.lfptofloat(in_string), to_float);


if __name__ == '__main__':
    unittest.main()



from waflib.TaskGen import feature, after_method


@feature("rtems_trace")
@after_method('apply_link')
def rtems_trace(self):
    if self.env.RTEMS_TEST_ENABLE:
        self.link_task.env.LINK_CC = self.env.BIN_RTEMS_TLD \
            + self.env.RTEMS_TEST_FLAGS + ['--']
